namespace eval ::tcl::tm {
    variable paths {}
    set pkgpattern {^([_[:alpha:]][:_[:alnum:]]*)-([[:digit:]].*)[.]tm$}
    namespace export path
    namespace ensemble create -command path -subcommands {add remove list}
}
proc ::tcl::tm::add {args} {
    variable paths
    set newpaths $paths
    foreach p $args {
	if {$p in $newpaths} {
	    continue
	}
	set pos [lsearch -glob $newpaths ${p}/*]
	if {$pos >= 0} {
	    return -code error \
		"$p is ancestor of existing module path [lindex $newpaths $pos]."
	}
	foreach ep $newpaths {
	    if {[string match ${ep}/* $p]} {
		return -code error \
		    "$p is subdirectory of existing module path $ep."
	    }
	}
	set newpaths [linsert $newpaths 0 $p]
    }
    set paths $newpaths
    return
}
proc ::tcl::tm::remove {args} {
    variable paths
    foreach p $args {
	set pos [lsearch -exact $paths $p]
	if {$pos >= 0} {
	    set paths [lreplace $paths $pos $pos]
	}
    }
}
proc ::tcl::tm::list {} {
    variable paths
    return  $paths
}
proc ::tcl::tm::UnknownHandler {original name args} {
    variable paths
    variable pkgpattern
    if {[llength $paths]} {
	set pkgpath [string map {:: /} $name]
	set pkgroot [file dirname $pkgpath]
	if {$pkgroot eq "."} {
	    set pkgroot ""
	}
	set satisfied 0
	foreach path $paths {
	    if {![interp issafe] && ![file exists $path]} {
		continue
	    }
	    set currentsearchpath [file join $path $pkgroot]
	    if {![interp issafe] && ![file exists $currentsearchpath]} {
		continue
	    }
	    set strip [llength [file split $path]]
	    catch {
		foreach file [glob -nocomplain -directory $currentsearchpath *.tm] {
		    set pkgfilename [join [lrange [file split $file] $strip end] ::]
		    if {![regexp -- $pkgpattern $pkgfilename --> pkgname pkgversion]} {
			continue
		    }
		    try {
			package vcompare $pkgversion 0
		    } on error {} {
			continue
		    }
		    if {[package ifneeded $pkgname $pkgversion] ne {}} {
			continue
		    }
		    package ifneeded $pkgname $pkgversion \
			"[::list package provide $pkgname $pkgversion];[::list source -encoding utf-8 $file]"
		    if {($pkgname eq $name)
			    && [package vsatisfies $pkgversion {*}$args]} {
			set satisfied 1
		    }
		}
	    }
	}
	if {$satisfied} {
	    return
	}
    }
    if {[llength $original]} {
	uplevel 1 $original [::linsert $args 0 $name]
    }
}
proc ::tcl::tm::Defaults {} {
    global env tcl_platform
    lassign [split [info tclversion] .] major minor
    set exe [file normalize [info nameofexecutable]]
    roots [::list \
	    [file dirname [info library]] \
	    [file join [file dirname [file dirname $exe]] lib] \
	    ]
    if {$tcl_platform(platform) eq "windows"} {
	set sep ";"
    } else {
	set sep ":"
    }
    for {set n $minor} {$n >= 0} {incr n -1} {
	foreach ev [::list \
			TCL${major}.${n}_TM_PATH \
			TCL${major}_${n}_TM_PATH \
        ] {
	    if {![info exists env($ev)]} continue
	    foreach p [split $env($ev) $sep] {
		path add $p
	    }
	}
    }
    return
}
proc ::tcl::tm::roots {paths} {
    regexp {^(\d+)\.(\d+)} [package present Tcl] - major minor
    foreach pa $paths {
	set p [file join $pa tcl$major]
	for {set n $minor} {$n >= 0} {incr n -1} {
	    set px [file join $p ${major}.${n}]
	    if {![interp issafe]} {set px [file normalize $px]}
	    path add $px
	}
	set px [file join $p site-tcl]
	if {![interp issafe]} {set px [file normalize $px]}
	path add $px
    }
    return
}
if {![interp issafe]} {::tcl::tm::Defaults}